﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

using G39DDCOOAPIWrapperSpace;
using System.Threading;

namespace G39DDCExample
{
    public class Wrapper
    {
        #region Members

        public G39DDCOOAPIWrapper apiWrapper = new G39DDCOOAPIWrapper();
        public G39DDCOOAPIWrapperDeviceInfo deviceInfo;
        public String deviceSerialNumber;

        public UInt32 apiStatus;

        public static String APIWRAPPERLIBRARY = G39DDCOOAPIWrapper.APILIBRARY;
        public static UInt32 APISTATUS_WRAPPERLIBRARY_LOADED = G39DDCOOAPIWrapper.STATUS_APILIBRARY_LOADED;
        public static UInt32 APISTATUS_WRAPPERLIBRARY_NOTFOUND = G39DDCOOAPIWrapper.STATUS_APILIBRARY_NOT_FOUND;

        public delegate void DeviceDDC2PreprocessedStreamReceivedHandler(object sender, EventArgs eventArgs);
        public event DeviceDDC2PreprocessedStreamReceivedHandler deviceDDC2PreprocessedStreamReceived;
        public delegate void DeviceAudioStreamReceivedHandler(object sender, EventArgs eventArgs);
        public event DeviceAudioStreamReceivedHandler deviceAudioStreamReceived;

        #endregion

        #region Public Methods

        public unsafe bool Init()
        {
            apiStatus = apiWrapper.GetStatus();
            if (apiStatus != APISTATUS_WRAPPERLIBRARY_LOADED)
            {
                return false;
            }

            if (apiWrapper.Enumerate() == 0)
            {
                return false;
            }

            if (apiWrapper.Open() == 0)
            {
                return false;
            }

            deviceInfo = new G39DDCOOAPIWrapperDeviceInfo();
            deviceInfo.serialNumber = apiWrapper.GetSerialNumber();
            deviceInfo.devicePath = apiWrapper.GetDevicePath();
            deviceInfo.interfaceType = apiWrapper.GetInterfaceType();

            // Set callback handlers
            G39DDCOOAPIWrapper.DDC2PreprocessedStreamCallback ddc2PreprocessedStreamCallback = OnDDC2PreprocessedStreamReceived;
            G39DDCOOAPIWrapper.AudioStreamCallback audioStreamCallback = OnAudioStreamReceived;
            apiWrapper.RegisterCallbacks(ddc2PreprocessedStreamCallback, audioStreamCallback);

            // Start example streaming
            // Start DDC1 streaming using ~50ms long buffers
            UInt32 aDDCTypeIndex;
            UInt32* aDDCTypeIndexPtr = &aDDCTypeIndex;
            G39DDCInfo aDDCInfo = new G39DDCInfo();
            apiWrapper.GetDDC1(0 /* Example */, aDDCTypeIndexPtr); // Get current DDC type index of DDC1 for channel 0
            apiWrapper.GetDDCInfo(aDDCTypeIndex, aDDCInfo); // Get DDC info for DDC type index
            apiWrapper.StartDDC1(0 /* Example */, CalculateSamplesPerBuffer(aDDCInfo.sampleRate, 50 /* Example time in ms */));

            // Start DDC2 streaming using ~50ms long buffers
            apiWrapper.GetDDC2(0 /* Example */, aDDCTypeIndexPtr); // Get current DDC type index of DDC2 for channel 0
            apiWrapper.GetDDCInfo(aDDCTypeIndex, aDDCInfo); // Get DDC info for DDC type index
            apiWrapper.StartDDC2(0 /* Example */, CalculateSamplesPerBuffer(aDDCInfo.sampleRate, 50 /* Example time in ms */));

            // Start audio streaming using ~50ms long buffers
            UInt32 anAudioSampleRateIndex, anAudioSampleRate;
            UInt32* anAudioSampleRateIndexPtr = &anAudioSampleRateIndex, anAudioSampleRatePtr = &anAudioSampleRate;
            apiWrapper.GetAudio(0 /* Example */, anAudioSampleRateIndexPtr); // Get current audio sample rate index
            apiWrapper.GetAudioSampleRate(anAudioSampleRateIndex, anAudioSampleRatePtr); // Get audio sample rate by its index
            apiWrapper.StartAudio(0 /* Example */, CalculateSamplesPerBuffer(anAudioSampleRate, 50 /* Example time in ms */));

            return true;
        }

        public bool IsDeviceConnected()
        {
            return apiWrapper.IsConnected() != 0;
        }

        public void CloseDevice()
        {
            apiWrapper.StopAudio(0 /* Example */);
            apiWrapper.StopDDC2(0 /* Example */);
            apiWrapper.StopDDC1(0 /* Example */);

            apiWrapper.Close();
        }

        public void SetFrequency(UInt64 theFrequency)
        {
            apiWrapper.SetFrequency(0 /* Example */, theFrequency);
        }

        public unsafe UInt64 GetFrequency()
        {
            UInt64 aFrequency = 0;
            UInt64* aFrequencyPtr = &aFrequency;
            apiWrapper.GetFrequency(0 /* Example */, aFrequencyPtr);

            return aFrequency;
        }

        public void SetVolume(Byte theVolume)
        {
            apiWrapper.SetVolume(0 /* Example */, theVolume);
        }

        public unsafe Byte GetVolume()
        {
            Byte aVolume = 0;
            Byte* aVolumePtr = &aVolume;
            apiWrapper.GetVolume(0 /* Example */, aVolumePtr);

            return aVolume;
        }

        #endregion

        #region Private Methods

        static UInt32 CalculateSamplesPerBuffer(UInt32 theSampleRate, UInt32 theTime)
        {
            UInt32 aSamplesPerBuffer = theTime * theSampleRate / 1000; // Calculte number of I/Q sample-sets per time (in ms) buffer
            return (UInt32)((aSamplesPerBuffer + 63) & (~63)); // Round up to 64
        }

        #endregion

        #region API Callback Handlers

        public unsafe void OnDDC2PreprocessedStreamReceived(UInt32 theChannel, float* theBuffer, UInt32 theNumberOfSamples, float theSLevelPeak, float theSLevelRms)
        {
            DDC2PreprocessedStreamEventArgs anEventArgs = new DDC2PreprocessedStreamEventArgs();
            anEventArgs.channel = theChannel;
            anEventArgs.numberOfSamples = theNumberOfSamples;
            anEventArgs.sLevelPeak = theSLevelPeak;
            anEventArgs.sLevelRms = theSLevelRms;
            anEventArgs.buffer = new float[theNumberOfSamples];
            // Copy buffer
            for (int anIx = 0; anIx < theNumberOfSamples; anIx++)
            {
                anEventArgs.buffer[anIx] = theBuffer[anIx];
            }

            // Fire event
            deviceDDC2PreprocessedStreamReceived(this, anEventArgs);
        }

        public unsafe void OnAudioStreamReceived(UInt32 theChannel, UInt32 theType, float* theBuffer, UInt32 theNumberOfSamples)
        {
            AudioStreamEventArgs anEventArgs = new AudioStreamEventArgs();
            anEventArgs.channel = theChannel;
            anEventArgs.type = theType;
            anEventArgs.numberOfSamples = theNumberOfSamples;
            anEventArgs.buffer = new float[2 * theNumberOfSamples];
            // Copy buffer
            for (int anIx = 0; anIx < 2 * theNumberOfSamples; anIx++)
            {
                anEventArgs.buffer[anIx] = theBuffer[anIx];
            }

            // Fire event
            deviceAudioStreamReceived(this, anEventArgs);
        }

        #endregion
    }
}
