// This is the main DLL file.

#include "stdafx.h"

#include "G39DDCOOAPIWrapper.h"

using namespace G39DDCOOAPIWrapperSpace;
using namespace System;
using namespace System::Runtime::InteropServices;


//
// Notification
//
void __stdcall G39DDCOOAPIWrapperNotification::RegisterCallbacks(DDC2_PREPROCESSED_STREAM_CALLBACK theDDC2PreprocessedStreamCallback, AUDIO_STREAM_CALLBACK theAudioStreamCallback) // Callbacks from unmanaged to managed C++
{
	ddc2PreprocessedStreamCallback = theDDC2PreprocessedStreamCallback;
	audioStreamCallback = theAudioStreamCallback;
}

void __stdcall G39DDCOOAPIWrapperNotification::G39DDC_IFCallback(IG39DDCDevice* theDevice, const SHORT* theBuffer, UINT32 theNumberOfSamples, UINT32 theCenterFrequency, WORD theAmplitude, UINT32 theADCSampleRate)
{
	// Not used in this example
}

void __stdcall G39DDCOOAPIWrapperNotification::G39DDC_DDC1StreamCallback(IG39DDCDevice* theDevice, UINT32 theChannel, const VOID* theBuffer, UINT32 theNumberOfSamples, UINT32 theBitsPerSample)
{
	// Not used in this example
}

BOOL __stdcall G39DDCOOAPIWrapperNotification::G39DDC_DDC1PlaybackStreamCallback(IG39DDCDevice* theDevice, UINT32 theChannel, VOID* theBuffer, UINT32 theNumberOfSamples, UINT32 theBitsPerSample)
{
	return FALSE; // Not used in this example
}

void __stdcall G39DDCOOAPIWrapperNotification::G39DDC_DDC2StreamCallback(IG39DDCDevice* theDevice, UINT32 theChannel, CONST FLOAT* theBuffer, UINT32 theNumberOfSamples)
{
	// Not used in this example
}

void __stdcall G39DDCOOAPIWrapperNotification::G39DDC_DDC2PreprocessedStreamCallback(IG39DDCDevice* theDevice, UINT32 theChannel, CONST FLOAT* theBuffer, UINT32 theNumberOfSamples, FLOAT theSLevelPeak, FLOAT theSLevelRms)
{
	ddc2PreprocessedStreamCallback(theDevice, theChannel, theBuffer, theNumberOfSamples, theSLevelPeak, theSLevelRms); // Callback to managed C++
}

void __stdcall G39DDCOOAPIWrapperNotification::G39DDC_AudioStreamCallback(IG39DDCDevice* theDevice, UINT32 theChannel, UINT32 theType, CONST FLOAT* theBuffer, UINT32 theNumberOfSamples)
{
	audioStreamCallback(theDevice, theChannel, theType, theBuffer, theNumberOfSamples); // Callback to managed C++
}

BOOL __stdcall G39DDCOOAPIWrapperNotification::G39DDC_AudioPlaybackStreamCallback(IG39DDCDevice* theDevice, UINT32 theChannel, FLOAT* theBuffer, UINT32 theNumberOfSamples)
{
	return FALSE; // Not used in this example
}

BOOL __stdcall G39DDCOOAPIWrapperNotification::G39DDC_SweepingCallback(IG39DDCDevice* theDevice, const SHORT* theBuffer, UINT32 theNumberOfSamples, UINT32 theCenterFrequency, UINT64 theFrequency)
{
	return FALSE; // Not used in this example
}


//
// ApiHandler
//
HMODULE ourModuleHandle = NULL;
G39DDCAPI_CREATE_INSTANCE ourCreateInstance;
IG39DDCDevice* ourDevice = NULL;
G39DDCOOAPIWrapperNotification ourNotification;

G39DDCOOAPIWrapper::G39DDCOOAPIWrapper()
{
	instance = this; // Used in callback from managed C++ to C#
	devicesCount = 0;

	IntPtr aPointer = Marshal::StringToHGlobalUni(APILIBRARY);
	ourModuleHandle = LoadLibrary((LPCWSTR)aPointer.ToPointer());

	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Failed to load {0}", APILIBRARY);
		status = STATUS_APILIBRARY_NOT_FOUND;
		return;
	}

    // Retrieve address of CreateInstance API function
    ourCreateInstance = (G39DDCAPI_CREATE_INSTANCE)GetProcAddress(ourModuleHandle, "CreateInstance");

	if (!ourCreateInstance)
	{
		Console::WriteLine("Incorrect {0}", APILIBRARY);
		FreeLibrary(ourModuleHandle);
		ourModuleHandle = NULL;
		status = STATUS_APILIBRARY_INCORRECT;
		return;
	}

	Console::WriteLine("{0} loaded", APILIBRARY);

	status = STATUS_APILIBRARY_LOADED;
}

G39DDCOOAPIWrapper::~G39DDCOOAPIWrapper()
{
	FreeLibrary(ourModuleHandle);
}

DWORD G39DDCOOAPIWrapper::GetStatus()
{
	return status;
}

BOOL G39DDCOOAPIWrapper::IsConnected()
{
	if (ourDevice == NULL)
	{
		return FALSE;
	}
	else
	{
		return ourDevice->IsConnected();
	}
}

String^ G39DDCOOAPIWrapper::GetDevicePath()
{
	return deviceInfo.devicePath;
}

BYTE G39DDCOOAPIWrapper::GetInterfaceType()
{
	return deviceInfo.interfaceType;
}

String^ G39DDCOOAPIWrapper::GetSerialNumber()
{
	return deviceInfo.serialNumber;
}

BOOL G39DDCOOAPIWrapper::Enumerate()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);
		return FALSE;
	}

	IG39DDCDeviceEnumerator* aDeviceEnumerator;
	G39DDC_DEVICE_INFO aDeviceInfo;

	BOOL aResult = FALSE;

	if (ourCreateInstance(G39DDC_CLASS_ID_DEVICE_ENUMERATOR, (void**)&aDeviceEnumerator))
	{
		aDeviceEnumerator->Enumerate();
		devicesCount = aDeviceEnumerator->GetCount();
        if (devicesCount > 0)
		{
			// Let's work with one device
			aDeviceEnumerator->GetDeviceInfo(0, &aDeviceInfo, sizeof(G39DDC_DEVICE_INFO));

			deviceInfo.devicePath = gcnew String(aDeviceInfo.DevicePath);
			deviceInfo.interfaceType = aDeviceInfo.InterfaceType;
			deviceInfo.serialNumber = gcnew String(aDeviceInfo.SerialNumber);
			deviceInfo.hwVersion = aDeviceInfo.HWVersion;
			deviceInfo.fwVersion = aDeviceInfo.FWVersion;
			deviceInfo.eepromVersion = aDeviceInfo.EEPROMVersion;
			deviceInfo.channelCount = aDeviceInfo.ChannelCount;
			deviceInfo.frontEndMinFrequency = aDeviceInfo.FrontEndMinFrequency;
			deviceInfo.frontEndMaxFrequency = aDeviceInfo.FrontEndMaxFrequency;
			deviceInfo.frontEndMaxHFFrequency = aDeviceInfo.FrontEndMaxHFFrequency;
			deviceInfo.frontEndFrequencyStep = aDeviceInfo.FrontEndFrequencyStep;
			deviceInfo.frontEndWindowWidth = aDeviceInfo.FrontEndWindowWidth;
			deviceInfo.hardwareOptions = aDeviceInfo.HardwareOptions;

			Console::WriteLine("Device #{0} found", deviceInfo.serialNumber);

			aResult = TRUE;
		}
		else
		{
			Console::WriteLine("No available device found");
		}

		aDeviceEnumerator->Release();
	}
    else
    {
        Console::WriteLine("Failed to create device enumerator object");
    }

	return aResult;
}

BOOL G39DDCOOAPIWrapper::Open()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);
		return FALSE;
	}

	if (devicesCount == 0)
	{
		Console::WriteLine("No available device found");
		return FALSE;
	}

    // Create instance of device object
    if (!ourCreateInstance(G39DDC_CLASS_ID_DEVICE,(void**)&ourDevice))
    {
		Console::WriteLine("Failed to create device object. Error code = {0}", GetLastError());
        return FALSE;
    }

	IntPtr aPointer = Marshal::StringToHGlobalAnsi(deviceInfo.devicePath);
	const char* aDevicePath = static_cast<char*>(aPointer.ToPointer());
    // Open device
    if (!ourDevice->Open(aDevicePath))
    {
        Console::WriteLine("Unable to open device");
        ourDevice->Release();
        FreeLibrary(ourModuleHandle);
        return FALSE;
    }
	Marshal::FreeHGlobal(aPointer);

	Console::WriteLine("Device opened");

	// Set power
    ourDevice->SetPower(TRUE);

	// Example initialization
    // Set DDC type of DDC1 (channel 0) to 12 (DDC Bandwidth = 320 kHz, SampleRate = 400 kHz, BitsPerSample = 32)
    // Number of available DDC types of DDC1 can be obtained using GetDDC1Count method
    ourDevice->SetDDC1(0, 12);

    // Set DDC type of DDC2 (channel 0) to 12 (DDC Bandwidth = 320 kHz, SampleRate = 400 kHz, BitsPerSample = 32)
    // Number of available DDC types of DDC2 (for current DDC type of DDC1) can be obtained using GetDDC2Count method
    ourDevice->SetDDC2(0, 12);

    // Set output audio sample rate index to 1 (48000 Hz)
    // Number of available audio sample rates can be obtained using GetAudioSampleRateCount
    ourDevice->SetAudio(0, 1);

    ourDevice->SetAGC(0, FALSE); // Disable AGC for channel 0

    ourDevice->SetDemodulatorMode(0, G39DDC_MODE_FMW); // Set FMW mode for channel 0

    ourDevice->SetDemodulatorFilterBandwidth(0, 250000); // Set demodulator filter bandwidth for channel 0 to 250 kHz

	return TRUE;
}

BOOL G39DDCOOAPIWrapper::Close()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	// Unregister all callback functions
    ourDevice->SetCallback(NULL);

    ourDevice->SetPower(FALSE);

	ourDevice->Close();
	ourDevice->Release();

	ourDevice = NULL;

	Console::WriteLine("Device closed");

	return TRUE;
}

BOOL G39DDCOOAPIWrapper::SetFrequency(UINT32 theChannel, UINT64 theFrequency)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->SetFrequency(theChannel, theFrequency);
}

BOOL G39DDCOOAPIWrapper::GetFrequency(UINT32 theChannel, UINT64* theFrequency)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->GetFrequency(theChannel, theFrequency);
}

BOOL G39DDCOOAPIWrapper::RegisterCallbacks(DDC2PreprocessedStreamCallback^ theDDC2PreprocessedStreamCallback, AudioStreamCallback^ theAudioStreamCallback)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	ddc2PreprocessedStreamCallback = theDDC2PreprocessedStreamCallback; // Callback to C#
	audioStreamCallback = theAudioStreamCallback; // Callback to C#
	ourDevice->SetCallback(&ourNotification);

	// DDC2PreprocessedStream-callback from unmanaged to managed C++
	NotificationDDC2PreprocessedStreamCallbackDelegate^ aDDC2PreprocessedStreamDelegate = gcnew NotificationDDC2PreprocessedStreamCallbackDelegate(NotificationDDC2PreprocessedStreamCallback);
	GCHandle aHandle = GCHandle::Alloc(aDDC2PreprocessedStreamDelegate);
	IntPtr aPointer = Marshal::GetFunctionPointerForDelegate(aDDC2PreprocessedStreamDelegate);
	DDC2_PREPROCESSED_STREAM_CALLBACK aDDC2PreprocessedStreamCallback = static_cast<DDC2_PREPROCESSED_STREAM_CALLBACK>(aPointer.ToPointer());

	// AudioStreamCallback from unmanaged to managed C++
	NotificationAudioStreamCallbackDelegate^ anAudioStreamCallbackDelegate = gcnew NotificationAudioStreamCallbackDelegate(NotificationAudioStreamCallback);
	aHandle = GCHandle::Alloc(anAudioStreamCallbackDelegate);
	aPointer = Marshal::GetFunctionPointerForDelegate(anAudioStreamCallbackDelegate);
	AUDIO_STREAM_CALLBACK anAudioStreamCallback = static_cast<AUDIO_STREAM_CALLBACK>(aPointer.ToPointer());

	// Force garbage collection cycle to prove that the delegate does not get disposed
	GC::Collect();

	// Register callbacks from unmanaged to managed C++
	ourNotification.RegisterCallbacks(aDDC2PreprocessedStreamCallback, anAudioStreamCallback);

	return TRUE;
}

BOOL G39DDCOOAPIWrapper::GetDDCInfo(UINT32 theDDCIndex, G39DDCInfo^ theDDCInfo)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	G39DDC_DDC_INFO aDDCInfo;
	if (!ourDevice->GetDDCInfo(theDDCIndex, &aDDCInfo))
	{
		Console::WriteLine("Getting DDC Info failed");

		return FALSE;
	}

	theDDCInfo->sampleRate = aDDCInfo.SampleRate;
	theDDCInfo->bandwidth = aDDCInfo.Bandwidth;
	theDDCInfo->bitsPerSample = aDDCInfo.BitsPerSample;

	return TRUE;
}

BOOL G39DDCOOAPIWrapper::GetDDC1(UINT32 theChannel, UINT32* theDDCIndex)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->GetDDC1(theChannel, theDDCIndex);
}

BOOL G39DDCOOAPIWrapper::StartDDC1(UINT32 theChannel, UINT32 theSamplesPerBuffer)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StartDDC1(theChannel, theSamplesPerBuffer);
}

BOOL G39DDCOOAPIWrapper::StopDDC1(UINT32 theChannel)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StopDDC1(theChannel);
}

BOOL G39DDCOOAPIWrapper::GetDDC2(UINT32 theChannel, UINT32* theDDCIndex)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->GetDDC2(theChannel, theDDCIndex);
}

BOOL G39DDCOOAPIWrapper::StartDDC2(UINT32 theChannel, UINT32 theSamplesPerBuffer)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StartDDC2(theChannel, theSamplesPerBuffer);
}

BOOL G39DDCOOAPIWrapper::StopDDC2(UINT32 theChannel)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StopDDC2(theChannel);
}

BOOL G39DDCOOAPIWrapper::GetAudio(UINT32 theChannel, UINT32* theSampleRateIndex)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->GetAudio(theChannel, theSampleRateIndex);
}

BOOL G39DDCOOAPIWrapper::GetAudioSampleRate(UINT32 theSampleRateIndex, UINT32* theSampleRate)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->GetAudioSampleRate(theSampleRateIndex, theSampleRate);
}

BOOL G39DDCOOAPIWrapper::StartAudio(UINT32 theChannel, UINT32 theSamplesPerBuffer)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StartAudio(theChannel, theSamplesPerBuffer);
}

BOOL G39DDCOOAPIWrapper::StopAudio(UINT32 theChannel)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StopAudio(theChannel);
}

BOOL G39DDCOOAPIWrapper::SetVolume(UINT32 theChannel, BYTE theVolume)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->SetVolume(theChannel, theVolume);
}

BOOL G39DDCOOAPIWrapper::GetVolume(UINT32 theChannel, BYTE* theVolume)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->GetVolume(theChannel, theVolume);
}

void G39DDCOOAPIWrapper::NotificationDDC2PreprocessedStreamCallback(IG39DDCDevice* theDevice, UINT32 theChannel, const FLOAT* theBuffer, UINT32 theNumberOfSamples, FLOAT theSLevelPeak, FLOAT theSLevelRMS)
{
	// Callback to C#
	instance->ddc2PreprocessedStreamCallback(theChannel, theBuffer, theNumberOfSamples, theSLevelPeak, theSLevelRMS);
}

void G39DDCOOAPIWrapper::NotificationAudioStreamCallback(IG39DDCDevice* theDevice, UINT32 theChannel, UINT32 theType, const FLOAT* theBuffer, UINT32 theNumberOfSamples)
{
	// In example only buffers of final audio samples are used (type = 2) (audio gain, audio filter and volume are applied)
	if (theType == 2)
	{
		// Callback to C#
		instance->audioStreamCallback(theChannel, theType, theBuffer, theNumberOfSamples);
	}
}