#include "G39DDCAPI.h"

#include <stdio.h>

#pragma once

using namespace System;

typedef void (*IF_CALLBACK)(IG39DDCDevice* device, const SHORT* buffer, UINT32 numberOfSamples, UINT32 centerFrequency, WORD amplitude, UINT32 adcSampleRate);
typedef void (*DDC1_STREAM_CALLBACK)(IG39DDCDevice* device, UINT32 channel, const VOID* buffer, UINT32 numberOfSamples, UINT32 bitsPerSample);
typedef BOOL (*DDC1_PLAYBACK_STREAM_CALLBACK)(IG39DDCDevice* Device, UINT32 channel, VOID* buffer, UINT32 numberOfSamples, UINT32 bitsPerSample);
typedef void (*DDC2_STREAM_CALLBACK)(IG39DDCDevice* device, UINT32 channel, CONST FLOAT* buffer, UINT32 numberOfSamples);
typedef void (*DDC2_PREPROCESSED_STREAM_CALLBACK)(IG39DDCDevice* device, UINT32 channel, CONST FLOAT* buffer, UINT32 numberOfSamples, FLOAT sLevelPeak, FLOAT sLevelRms);
typedef void (*AUDIO_STREAM_CALLBACK)(IG39DDCDevice* device, UINT32 channel, UINT32 type, CONST FLOAT* buffer, UINT32 numberOfSamples);
typedef BOOL (*AUDIO_PLAYBACK_STREAM_CALLBACK)(IG39DDCDevice* device, UINT32 channel, FLOAT* buffer, UINT32 numberOfSamples);
typedef BOOL (*SWEEPING_CALLBACK)(IG39DDCDevice* device, const SHORT* buffer, UINT32 numberOfSamples, UINT32 centerFrequency, UINT64 frequency);

namespace G39DDCOOAPIWrapperSpace
{
	//
	// Notification (callbacks from unmanaged to managed C++)
	//
	public class G39DDCOOAPIWrapperNotification : public IG39DDCDeviceCallback
	{
		public:

			void __stdcall RegisterCallbacks(DDC2_PREPROCESSED_STREAM_CALLBACK ddc2PreprocessedStreamCallback, AUDIO_STREAM_CALLBACK audioStreamCallback); // Callbacks from unmanaged to managed C++

			void __stdcall G39DDC_IFCallback(IG39DDCDevice* device, const SHORT* buffer, UINT32 numberOfSamples, UINT32 centerFrequency, WORD amplitude, UINT32 adcSampleRate);
			void __stdcall G39DDC_DDC1StreamCallback(IG39DDCDevice* device, UINT32 channel, const VOID* buffer, UINT32 numberOfSamples, UINT32 bitsPerSample);
			BOOL __stdcall G39DDC_DDC1PlaybackStreamCallback(IG39DDCDevice* Device, UINT32 channel, VOID* buffer, UINT32 numberOfSamples, UINT32 bitsPerSample);
			void __stdcall G39DDC_DDC2StreamCallback(IG39DDCDevice* device, UINT32 channel, CONST FLOAT* buffer, UINT32 numberOfSamples);
			void __stdcall G39DDC_DDC2PreprocessedStreamCallback(IG39DDCDevice* device, UINT32 channel, CONST FLOAT* buffer, UINT32 numberOfSamples, FLOAT sLevelPeak, FLOAT sLevelRms);
			void __stdcall G39DDC_AudioStreamCallback(IG39DDCDevice* device, UINT32 channel, UINT32 type, CONST FLOAT* buffer, UINT32 numberOfSamples);
			BOOL __stdcall G39DDC_AudioPlaybackStreamCallback(IG39DDCDevice* device, UINT32 channel, FLOAT* buffer, UINT32 numberOfSamples);
			BOOL __stdcall G39DDC_SweepingCallback(IG39DDCDevice* device, const SHORT* buffer, UINT32 numberOfSamples, UINT32 centerFrequency, UINT64 frequency);


		private:

			// Callbacks to managed C++
			IF_CALLBACK ifCallback; // Not used in this example
			DDC1_STREAM_CALLBACK ddc1StreamCallback; // Not used in this example
			DDC1_PLAYBACK_STREAM_CALLBACK ddc1PlaybackStreamCallback; // Not used in this example
			DDC2_STREAM_CALLBACK ddc2StreamCallback; // Not used in this example 
			DDC2_PREPROCESSED_STREAM_CALLBACK ddc2PreprocessedStreamCallback;
			AUDIO_STREAM_CALLBACK audioStreamCallback;
			AUDIO_PLAYBACK_STREAM_CALLBACK audioPlaybackStreamCallback; // Not used in this example 
			SWEEPING_CALLBACK sweepingCallback; // Not used in this example 
	};

	//
	// Device info
	//
	public ref class G39DDCOOAPIWrapperDeviceInfo
	{
		public:

			String^ devicePath;
			BYTE interfaceType;
			String^ serialNumber;
			UINT16 hwVersion;
			UINT16 fwVersion;
			BYTE eepromVersion;
			UINT32 channelCount;
			UINT64 frontEndMinFrequency;
			UINT64 frontEndMaxFrequency;
			UINT64 frontEndMaxHFFrequency;
			UINT32 frontEndFrequencyStep;
			UINT32 frontEndWindowWidth;
			UINT32 hardwareOptions;
	};

	//
	// DDC Info
	//
	public ref class G39DDCInfo
	{
		public:

			UINT32 sampleRate;
			UINT32 bandwidth;
			UINT32 bitsPerSample;
	};


	//
	// API Wrapper
	//
	public ref class G39DDCOOAPIWrapper
	{
		public:

			static String^ APILIBRARY = gcnew String("G39DDCAPI.dll");

			static const DWORD STATUS_APILIBRARY_LOADED = 100;
			static const DWORD STATUS_APILIBRARY_NOT_FOUND = 200;
			static const DWORD STATUS_APILIBRARY_INCORRECT = 300;
			static const DWORD STATUS_DEVICE_CREATINGFAILED = 400;

			static const DWORD DEVICE_INTERFACE_PCI = 0;
			static const DWORD DEVICE_INTERFACE_USB = 1;

			G39DDCOOAPIWrapper();
			~G39DDCOOAPIWrapper();

			delegate void DDC2PreprocessedStreamCallback(UINT32 channel, const FLOAT* buffer, UINT32 numberOfSamples, FLOAT sLevelPeak, FLOAT sLevelRms); // Callback to C#
			delegate void AudioStreamCallback(UINT32 channel, UINT32 type, const FLOAT* buffer, UINT32 numberOfSamples); // Callback to C#

			static void NotificationDDC2PreprocessedStreamCallback(IG39DDCDevice* device, UINT32 channel, const FLOAT* buffer, UINT32 numberOfSamples, FLOAT sLevelPeak, FLOAT sLevelRms); // Callback for Notification (to unmanaged C++)
			delegate void NotificationDDC2PreprocessedStreamCallbackDelegate(IG39DDCDevice*, UINT32, const FLOAT*, UINT32, FLOAT, FLOAT); // Callback from Notification (from unmanaged C++)
			static void NotificationAudioStreamCallback(IG39DDCDevice* device, UINT32 channel, UINT32 type, const FLOAT* buffer, UINT32 numberOfSamples); // Callback for Notification (to unmanaged C++)
			delegate void NotificationAudioStreamCallbackDelegate(IG39DDCDevice*, UINT32, UINT32, const FLOAT*, UINT32); // Callback from Notification (from unmanaged C++)

			DWORD GetStatus();
			BOOL IsConnected();
			String^ GetDevicePath();
			BYTE GetInterfaceType();
			String^ GetSerialNumber();

			BOOL Enumerate();
			BOOL Open();
			BOOL Close();
			BOOL SetFrequency(UINT32 channel, UINT64 frequency);
			BOOL GetFrequency(UINT32 channel, UINT64* frequency);
			BOOL RegisterCallbacks(DDC2PreprocessedStreamCallback^ ddc2PreprocessedStreamCallback, AudioStreamCallback^ audioStreamCallback);
			BOOL GetDDCInfo(UINT32 ddcIndex, G39DDCInfo^ ddcInfo);
			BOOL GetDDC1(UINT32 channel, UINT32* ddcIndex);
			BOOL StartDDC1(UINT32 channel, UINT32 samplesPerBuffer);
			BOOL StopDDC1(UINT32 channel);
			BOOL GetDDC2(UINT32 channel, UINT32* ddcIndex);
			BOOL StartDDC2(UINT32 channel, UINT32 samplesPerBuffer);
			BOOL StopDDC2(UINT32 channel);
			BOOL GetAudio(UINT32 channel, UINT32* sampleRateIndex);
			BOOL GetAudioSampleRate(UINT32 sampleRateIndex, UINT32* sampleRate);
			BOOL StartAudio(UINT32 channel, UINT32 samplesPerBuffer);
			BOOL StopAudio(UINT32 channel);
			BOOL SetVolume(UINT32 channel, BYTE volume);
			BOOL GetVolume(UINT32 channel, BYTE* volume);

		private:

			static G39DDCOOAPIWrapper^ instance;
			DWORD status;

			int devicesCount;

			G39DDCOOAPIWrapperDeviceInfo deviceInfo;

			DDC2PreprocessedStreamCallback^ ddc2PreprocessedStreamCallback;
			AudioStreamCallback^ audioStreamCallback;
	};
}